# idedither - generate combined image from dither
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 19. Aug. 2000
##############################################################################
procedure idedither(input,output)

string	input	{prompt="List of dither images"}
string	output	{prompt="Name of output file"}
string	match	{prompt="Method used to match images (wcs|manual|pick1)"}
bool	subsky	{prompt="Subtract sky"}
string	scale	{prompt="Scale images (none|mode|median|mean|<keyword>)"}
bool	check	{prompt="Check range"}
bool	verbose	{prompt="Verbose"}
string	tmpdir	{prompt="Temporary directory for file conversion"}

string *list1	{prompt="Ignore this parameter(list1)"}
string *list2	{prompt="Ignore this parameter(list2)"}

begin
  string	ini,ino
  string	logf,fname,diffile,skysub,key,offset,junk
  struct	cmd
  file		tmpl,inli,deli,sky,tf1,tf2,tcent
  real		x,y,xr,yr,ref_ra,ref_dec,tel_ra,tel_dec,xoff,yoff,imscale
  real		xmax,xmin,ymax,ymin
  int		wcs,nsc
  bool		first

  # scale in arcsec/pixel
  imscale = ingrid_ql.scale

  # remove any temporary files that could have been left around by a previous
  # aborted run
  delete("tmp$idedither*",yes,verify=no)

  # copy input arguments to force definition at the beginning of the script
  ini = input
  ino = output

  # decide if to show output
  if ( verbose )
    logf = "STDOUT"
  else
    logf = ""

  # expand output image name.
  tmpl = mktemp("tmp$idedither")
  sections(ino,option="fullname",> tmpl)
  list1 = tmpl
  nsc = fscan(list1, fname)
  delete(tmpl,yes,verify=no)

  # check if output image exists
  imgaccess(fname, verbose=no)
  if ( imgaccess.exists )
    {
    beep
    print("")
    print("ERROR: operation would override output file "//fname)
    print("idedither aborted")
    print("")
    beep
    bye
    }
 
  # create unique files for expanding image names, etc.
  tmpl = mktemp("tmp$idedither")
  inli = mktemp("tmp$idedither")
  deli = mktemp("tmp$idedither")
  if ( match != "wcs" )
    {
    tf1   = mktemp("tmp$idedither")
    tf2   = mktemp("tmp$idedither")
    tcent = mktemp("tmp$idedither")
    }

  # expand input image names.
  sections(ini,option="fullname",> tmpl)
 
  # Work through list of input files and find minimum and maximum shifts
  # based on coordinate information as well as the frame size (from the
  # first image)
  if ( check && match != "wcs" )
    {
    if ( verbose ) print("finding minimum and maximum shifts ...")
    first = yes
    list1 = tmpl
    while (fscan(list1, fname) != EOF)
      {
      # find size of the frame for first (reference) image
      if ( first )
        {
        ifileinfo(input=fname,verbose=no)
        xmin = 1
        xmax = ifileinfo.naxis1
        ymin = 1
        ymax = ifileinfo.naxis2
        }

      # find position telescope was pointing at
      imgets (image=fname//"[0]", param="RA")
      tel_ra = real(imgets.value)
      imgets (image=fname//"[0]", param="DEC")
      tel_dec = real(imgets.value)
      imgets (image=fname//"[0]", param="XAPOFF")
      xoff = real(imgets.value)
      imgets (image=fname//"[0]", param="YAPOFF")
      yoff = real(imgets.value)
      tel_ra = tel_ra + xoff/15.
      tel_dec = tel_dec + yoff

      if ( first )
        {
        # store position telescope was pointing at as reference
        ref_ra = tel_ra
        ref_dec = tel_dec
        first = no
        }
      else
        {
        # conversion factor degrees to radian: PI/180 = 0.017453293
        # conversion factor arcsec to hour   : 1 hour = 54000 arcsec
        # conversion factor arcsec to degree : 1 deg  = 3600 arcsec
        x = 54000.*cos(ref_dec*0.017453293)*(tel_ra - ref_ra)/imscale
        y = -3600.*(tel_dec - ref_dec)/imscale
        # check limits
        if ( x >= 0 && x > xmin ) xmin = x
        if ( x < 0  && 1024+x < xmax ) xmax = 1024+x
        if ( y >= 0 && y > ymin ) ymin = y
        if ( y < 0  && 1024+y < ymax ) ymax = 1024+y
        }
      }
    print ( "xmin = "//xmin )
    print ( "xmax = "//xmax )
    print ( "ymin = "//ymin )
    print ( "ymax = "//ymax )
    if ( xmin >= xmax || ymin >= ymax )
      {
      beep
      print("")
      print("ERROR: no common area among input files.")
      print('*****  Please re-run idedither with match="wcs"')
      print("idedither aborted")
      print("")
      beep
      bye
      }
    }

  # Work through list of input files and check if they are already pre read
  # subtracted images containing a wcs
  first = yes
  list1 = tmpl
  while (fscan(list1, fname) != EOF)
    {
    # get temporary file name
    diffile = mktemp (tmpdir//"idedither")

    # check file subtraction and format
    icheckfile(fname,verbose=no)

    # check if data file needs to be post-pre subtracted
    if ( ! icheckfile.subtract )
      {
      # calculate post-pre image
      print ("calculating post-pre for image "//fname//" ...")
      iframediff (input = fname,
                  output = diffile,
                  wcs = yes,
                  verbose = no,
                  info = no )
      }
    else
      {
      imcopy (input = fname,
              output = diffile,
              verbose = verbose)
      }

    print(osfn(diffile), >> inli)
    print(osfn(diffile), >> deli)

    # check if we need to get the offset manually
    # if we do, we need to display the frame, read the image cursor,
    # centroid and remember the coordinates for input to imcombine
    if ( match == "manual" )
      {
      display(diffile,1)
      if ( first )
        print " *** please mark one star in the image with 'm'"
      else
        print " *** please mark the same star in this image with 'm'"

      key = ""
      while ( key != "m" )
        {
        imarkstar(input=diffile,
                  center="centroid",
                  verbose=yes)
        key = imarkstar.key

        if ( key == "m" )
          {
          if (first)
            {
            xr = imarkstar.x
            yr = imarkstar.y
            }
          print((xr-imarkstar.x)//" "//(yr-imarkstar.y), >> tf2)
          break
          }
        else
          beep
        }

      }

    # check if we only need to get the first offset manually
    # if we do, we need to display the frame, read the image cursor,
    # centroid and remember the coordinates for input to imcombine.
    # For all the other images we need to calculate the shift from the offset
    # and then centroid for better accuracy.
    if ( match == "pick1" )
      {
      # find position telescope was pointing at
      imgets (image=diffile, param="RA")
      tel_ra = real(imgets.value)
      imgets (image=diffile, param="DEC")
      tel_dec = real(imgets.value)
      imgets (image=diffile, param="XAPOFF")
      xoff = real(imgets.value)
      imgets (image=diffile, param="YAPOFF")
      yoff = real(imgets.value)
      tel_ra = tel_ra + xoff/15.
      tel_dec = tel_dec + yoff

      if ( first )
        {
        # store position telescope was pointing at as reference
        ref_ra = tel_ra
        ref_dec = tel_dec

        # display image
        display(diffile,1)
        # mark area for star to be picked
        if ( check )
          {
          x = xmin + (xmax - xmin) / 2.
          y = ymin + (ymax - ymin) / 2.
          print(int(x)//" "//int(y)) | tvmark (frame=1,\
                coords="STDIN",\
                mark = "rectangle",\
                lengths = int(xmax - xmin)//" "//(ymax-ymin)/(xmax-xmin),\
                color = 205,\
                interactive = no )
          }
        # print usage information
        print " *** please mark one star in the image with 'm'"
        if ( check ) print " *** within the green rectangle (common area)"

        key = ""
        while ( key != "m" )
          {
          imarkstar(input=diffile,
                    center="centroid",
                    verbose=yes)

          key = imarkstar.key
          if ( key == "m" )
            {
            xr = imarkstar.x
            yr = imarkstar.y
            print("0 0", >> tf2)
            break
            }
          else
            beep
          }
        }
      else
        {
        # conversion factor degrees to radian: PI/180 = 0.017453293
        # conversion factor arcsec to hour   : 1 hour = 54000 arcsec
        # conversion factor arcsec to degree : 1 deg  = 3600 arcsec
        x = xr + 54000.*cos(ref_dec*0.017453293)*(tel_ra - ref_ra)/imscale
        y = yr - 3600.*(tel_dec - ref_dec)/imscale

        # round to three decimal places
        x = real (nint (x * 1000.)) / 1000.
        y = real (nint (y * 1000.)) / 1000.

        print("Auto     : x="//x//" y="//y)
        delete(tf1,yes,verify=no, >>& "/dev/null")
        print(x//" "//y, > tf1)

        imcentroid(input=diffile,
                   reference="",
                   coords=tf1,
                   shifts="",
                   boxsize=7,
                   bigbox=11,
                   negative=no,
                   background=INDEF,
                   lower=INDEF,
                   upper=INDEF,
                   niterate=3,
                   tolerance=0,
                   verbose=no, > tcent)

        # read centroiding information
        list2 = tcent
        nsc = fscan (list2)
        nsc = fscan (list2, junk, x, junk, y)
        list2 = ""
        delete(tcent,yes,verify=no)

        print("Centroid : x="//x//" y="//y)
        print((xr-x)//" "//(yr-y), >> tf2)
        }

      }

    first = no
    }

  if ( subsky )
    {
    # calculate sky = median of unshifted images
    if ( verbose ) print "calculating sky ...."
    sky = mktemp("tmp$sky")
    imcombine (\
          input = "@"//osfn(inli),
         output = sky,
        rejmask = "",
         plfile = "",
          sigma = "",
        logfile = logf,
        combine = "median",
#         reject = "sigclip",
         reject = "none",
        project = no,
        outtype = "real",
        offsets = "",
       masktype = "none",
      maskvalue = 0.,
          blank = 1.,
#          scale = "none",
          scale = scale,
           zero = "none",
         weight = "none",
        statsec = "",
        expname = "EXPTIM",
     lthreshold = INDEF,
     hthreshold = INDEF,
           nlow = 1,
          nhigh = 1,
          nkeep = 1,
          mclip = yes,
         lsigma = 3.,
         hsigma = 3.,
        rdnoise = "0.",
           gain = "1.",
         snoise = "0.",
       sigscale = 0.1,
          pclip = -0.5,
           grow = 0.)

    # get temporary file name
    skysub = mktemp (tmpdir//"idedither")

    # Work through list of images and do sky subtraction
    if ( verbose ) print "calculating image-sky ...."
    list1 = inli
    while (fscan(list1, fname) != EOF)
      {

      imrename (oldnames = fname,
                newnames = skysub,
                verbose = verbose)

      imarith (operand1 = skysub,
               op = "-",
               operand2 = sky,
               result = fname,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = verbose,
               noact = no)
 
      imdelete(skysub,yes,verify=no)
      }

    imdelete(sky,yes,verify=no)
    }

  # define offset parameter
  if ( match == "wcs" )
    offset = "wcs"
  else
    offset = tf2

  # unshift and coadd images
  if ( verbose ) print "coadding images ...."
  imcombine (\
        input = "@"//osfn(inli),
       output = ino,
      rejmask = "",
       plfile = "",
        sigma = "",
      logfile = logf,
      combine = "average",
       reject = "sigclip",
      project = no,
      outtype = "real",
      offsets = offset,
     masktype = "goodvalue",
    maskvalue = 0.,
        blank = 1.,
#        scale = "exposure",
        scale = "none",
         zero = scale,
       weight = "none",
      statsec = "[100:200,100:200]",
      expname = "EXPTIM",
   lthreshold = INDEF,
   hthreshold = INDEF,
         nlow = 1,
        nhigh = 1,
        nkeep = 1,
        mclip = yes,
       lsigma = 3.,
       hsigma = 3.,
      rdnoise = "0.",
         gain = "1.",
       snoise = "0.",
     sigscale = 0.1,
        pclip = -0.5,
         grow = 0.)

  imdelete("@"//deli,yes,verify=no)
  delete(tmpl,yes,verify=no)
  delete(inli,yes,verify=no)
  delete(deli,yes,verify=no)
  if ( match == "man" )
    {
    delete(tf1,yes,verify=no)
    delete(tf2,yes,verify=no)
    }

  # clear lists
  list1 = ""
  list2 = ""

end
